﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using MorganStanley.Taser.Common;
using MorganStanley.Taser.Common.Limits;

namespace MorganStanley.Taser.Views.Views.Approvals.LimitMonitor.LimitSetupView.LimitTypeGrid
{
  public class LimitMonitorSetupRecordPresenter
  {
    private string _currency;

    public string Currency
    {
      get { return _currency; }
      set { _currency = value; }
    }

    public string ProductTypesAsString
    {
      get
      {
        string str = string.Empty;
        foreach (Enums.ProductType productType in ProductTypes)
        {
          str += productType.ToString() + ", ";
        }
        str = str.Substring(0, str.Length - 2);
        return str;
      }
    }

    private IList<Enums.ProductType> _productTypes;

    public IList<Enums.ProductType> ProductTypes
    {
      get {
        if (_productTypes == null)
          _productTypes = new List<Enums.ProductType>();
        return _productTypes; }
      set { _productTypes = value; }
    }
    private IList<LimitSetup> _limitSetups_Net;

    public IList<LimitSetup> LimitSetups_Net
    {
      get
      {
        if (_limitSetups_Net == null)
          _limitSetups_Net = new List<LimitSetup>();
        return _limitSetups_Net;
      }
    }

    private IList<LimitSetup> _limitSetups_Gross;
    public IList<LimitSetup> LimitSetups_Gross
    {
      get
      {
        if (_limitSetups_Gross == null)
          _limitSetups_Gross = new List<LimitSetup>();
        return _limitSetups_Gross;
      }
    }

    public IList<LimitSetup> LimitSetups
    {
      get
      {
        List<LimitSetup> limitSetups = new List<LimitSetup>(); ;
        if (_limitSetups_Net != null) 
          limitSetups.AddRange(_limitSetups_Net);
        if (_limitSetups_Gross != null) 
          limitSetups.AddRange(_limitSetups_Gross);
        return limitSetups;
      }
    }


    private IList<LimitMaturity> _limitMaturity;


    public IList<LimitMaturity> LimitMaturity
    {
      get {
        if (_limitMaturity == null)
          _limitMaturity = new List<LimitMaturity>();  
        return _limitMaturity; }
    }

    public void AddLimitSetup(LimitSetup limitSetup)
    {
      if (limitSetup.Definition.Net)
      {
        if (!LimitSetups_Net.Contains(limitSetup))
        {
          LimitSetups_Net.Add(limitSetup);
        }
      }
      else
      {
        if (!LimitSetups_Gross.Contains(limitSetup))
        {
          LimitSetups_Gross.Add(limitSetup);
        }  
      }
      
      if (!LimitMaturity.Contains(limitSetup.Definition.Maturity))
      {
        LimitMaturity.Add(limitSetup.Definition.Maturity);
      }
    }
    
  }


  public class LimitMonitorSetupRecordPresenterCollection : ObservableCollection<LimitMonitorSetupRecordPresenter>
  {
    public void AddRangeRecordPresenter(IEnumerable<LimitMonitorSetupRecordPresenter> limitMonitorSetupRecordPresenters)
    {
      foreach (LimitMonitorSetupRecordPresenter limitMonitorSetupRecordPresenter in limitMonitorSetupRecordPresenters)
      {
        Items.Add(limitMonitorSetupRecordPresenter);
      }
    }

    public void RemoveRangeRecordPresenter(IEnumerable<LimitMonitorSetupRecordPresenter> limitMonitorSetupRecordPresenters)
    {
      foreach (LimitMonitorSetupRecordPresenter limitMonitorSetupRecordPresenter in limitMonitorSetupRecordPresenters)
      {
        Items.Remove(limitMonitorSetupRecordPresenter);
      }
    }

    public bool ContainLimitSetup(LimitSetup limitSetup)
    {
      foreach (LimitMonitorSetupRecordPresenter limitMonitorSetupRecordPresenter in Items)
      {
        bool productsEqual = true;
        if (limitMonitorSetupRecordPresenter.ProductTypes.Count == limitSetup.ProductTypes.Count)
        {
          foreach (Enums.ProductType productType in limitSetup.ProductTypes)
          {
            if (!limitMonitorSetupRecordPresenter.ProductTypes.Contains(productType))
            {
              productsEqual = false;
              break;
            }
          }
        }
        if (limitMonitorSetupRecordPresenter.Currency == limitSetup.Currency && productsEqual)
        {
          return true;
        }
      }
      return false;
    }

    public LimitMonitorSetupRecordPresenter FindPresenterForLimitSetup(LimitSetup limitSetup)
    {
      foreach (LimitMonitorSetupRecordPresenter limitMonitorSetupRecordPresenter in Items)
      {
        bool productsEqual = true;
        if (limitMonitorSetupRecordPresenter.ProductTypes.Count == limitSetup.ProductTypes.Count)
        {
          foreach (Enums.ProductType productType in limitSetup.ProductTypes)
          {
            if (!limitMonitorSetupRecordPresenter.ProductTypes.Contains(productType))
            {
              productsEqual = false;
              break;
            }
          }
        }
        if (limitMonitorSetupRecordPresenter.Currency == limitSetup.Currency && productsEqual)
        {
          return limitMonitorSetupRecordPresenter;
        }
      }
      return null;
    }

    public IList<LimitSetup> FindLimitSetupsForMaturity(string maturity, bool? net)
    {
      IList<LimitSetup> limitSetups = new List<LimitSetup>();
      foreach (LimitMonitorSetupRecordPresenter limitMonitorSetupRecordPresenter in Items)
      {
        if (net == null)
        {
          for (int i = 0; i < limitMonitorSetupRecordPresenter.LimitSetups.Count; i++)
          {
            if (limitMonitorSetupRecordPresenter.LimitSetups[i].Definition.Maturity.ToString().Equals(maturity))
              limitSetups.Add(limitMonitorSetupRecordPresenter.LimitSetups[i]);
          }
        }
        else if ((bool)net)
        {
          for (int i = 0; i < limitMonitorSetupRecordPresenter.LimitSetups_Net.Count; i++)
          {
            if (limitMonitorSetupRecordPresenter.LimitSetups_Net[i].Definition.Maturity.ToString().Equals(maturity))
              limitSetups.Add(limitMonitorSetupRecordPresenter.LimitSetups_Net[i]);
          }
        }
        else if (!(bool)net)
        {
          for (int i = 0; i < limitMonitorSetupRecordPresenter.LimitSetups_Gross.Count; i++)
          {
            if (limitMonitorSetupRecordPresenter.LimitSetups_Gross[i].Definition.Maturity.ToString().Equals(maturity))
              limitSetups.Add(limitMonitorSetupRecordPresenter.LimitSetups_Gross[i]);
          }
        }
      }
      return limitSetups;
    }

    public void RemoveLimitSetupsForMaturity(string maturity, bool? net)
    {
      foreach (LimitMonitorSetupRecordPresenter limitMonitorSetupRecordPresenter in Items)
      {
        if (net == null)
        {
          for (int i = 0; i < limitMonitorSetupRecordPresenter.LimitSetups.Count; i++)
          {
            if (limitMonitorSetupRecordPresenter.LimitSetups[i].Definition.Maturity.ToString().Equals(maturity))
              limitMonitorSetupRecordPresenter.LimitSetups.RemoveAt(i);
          }
        }
        else if ((bool)net)
        {
          for (int i = 0; i < limitMonitorSetupRecordPresenter.LimitSetups_Net.Count; i++)
          {
            if (limitMonitorSetupRecordPresenter.LimitSetups_Net[i].Definition.Maturity.ToString().Equals(maturity))
              limitMonitorSetupRecordPresenter.LimitSetups_Net.RemoveAt(i);
          }
        }
        else if (!(bool)net)
        {
          for (int i = 0; i < limitMonitorSetupRecordPresenter.LimitSetups_Gross.Count; i++)
          {
            if (limitMonitorSetupRecordPresenter.LimitSetups_Gross[i].Definition.Maturity.ToString().Equals(maturity))
              limitMonitorSetupRecordPresenter.LimitSetups_Gross.RemoveAt(i);
          }
        }
      }
    }

    public  IList<LimitSetup> GetLimitSetups()
    {
      List<LimitSetup> limitSetups = new List<LimitSetup>();
      foreach (LimitMonitorSetupRecordPresenter limitMonitorSetupRecordPresenter in Items)
      {
        limitSetups.AddRange(limitMonitorSetupRecordPresenter.LimitSetups);
      }
      return limitSetups;
    }

  }


  internal class LimitMonitorSetupRecordPresenterFactory
  {
    private IList<LimitSetup> _limitsetups;
    public LimitMonitorSetupRecordPresenterFactory(IList<LimitSetup> limitSetups)
    {
      _limitsetups = limitSetups;
    }

    public LimitMonitorSetupRecordPresenterCollection GenerateRecords()
    {
      LimitMonitorSetupRecordPresenterCollection limitMonitorSetupRecordPresenters = new LimitMonitorSetupRecordPresenterCollection();
      foreach (LimitSetup limitSetup in _limitsetups)
      {
        if (limitSetup.Definition.LimitSpecified)
        {
          if (!limitMonitorSetupRecordPresenters.ContainLimitSetup(limitSetup))
          {
            LimitMonitorSetupRecordPresenter limitMonitorSetupRecordPresenter = new LimitMonitorSetupRecordPresenter();
            limitMonitorSetupRecordPresenter.Currency = limitSetup.Currency;
            limitMonitorSetupRecordPresenter.ProductTypes = limitSetup.ProductTypes;
            limitMonitorSetupRecordPresenter.AddLimitSetup(limitSetup);
            limitMonitorSetupRecordPresenters.Add(limitMonitorSetupRecordPresenter);
          }
          else
          {
            LimitMonitorSetupRecordPresenter limitMonitorSetupRecordPresenter =
              limitMonitorSetupRecordPresenters.FindPresenterForLimitSetup(limitSetup);
            if (limitMonitorSetupRecordPresenter != null)
              limitMonitorSetupRecordPresenter.AddLimitSetup(limitSetup);
          }
        }
      }
      return limitMonitorSetupRecordPresenters;
    }

  }
}
